@@ -13,6 +13,9 @@ from typing import Callable, ClassVar, Dict, List, Tuple, Union  # noqa
 # To help us generate the right parameter string from a dict of parameters
 from urllib.parse import urlencode
 
+# To be able to deprecate methods
+import warnings
+
 
 class AbstractEndpoint(metaclass=ABCMeta):
     """
@@ -51,7 +54,8 @@ class AbstractEndpoint(metaclass=ABCMeta):
 
     def __init__(self):
         # A mapping of 'parameter' strings
-        self.parameter_name_to_available_Parameters: Dict[str, Callable[[Union[str, int, bool]], CCParameter.AbstractParameter]] = \
+        self.parameter_name_to_available_Parameters: Dict[
+            str, Callable[[Union[str, int, bool]], CCParameter.AbstractParameter]] = \
             {ap.name: ap for ap in self.available_Parameters}
 
         # Stores the instantiated Parameters for this Endpoint.
@@ -132,17 +136,13 @@ class AbstractEndpoint(metaclass=ABCMeta):
         elif parameter_name in self.parameter_name_to_value:
             self.parameter_name_to_value.pop(parameter_name)
 
-    def compose_endpoint(self) -> str:
+    def get_composed_params(self) -> Dict[str, Union[bool, int, str]]:
         """
-        Returns a valid URL suffix of this endpoint and any parameters
-        that have been defined for it.
+        Returns a requests-compatible dictionary of this Endpoint's current parameters.
 
-        Note that the ordering of the parameters is not guaranteed.
-
-        :return: A string like:
-            'rebalance?dryrun=false&allow_capacity_estimation=false&concurrent_partition_movements_per_broker=5'
-            'state'
-            'stop_proposal_execution'
+        :return: A dict like:
+            {'json': True,
+             'allow_capacity_estimation': False}
         """
         # All parameter:value mappings that have been specified for this endpoint.
         #
@@ -158,6 +158,27 @@ class AbstractEndpoint(metaclass=ABCMeta):
         if self.parameter_name_to_value:
             combined_parameter_to_value.update(self.parameter_name_to_value)
 
+        return combined_parameter_to_value
+
+    def compose_endpoint(self) -> str:
+        """
+        Returns a valid URL suffix of this endpoint and any parameters
+        that have been defined for it.
+
+        Note that the ordering of the parameters is not guaranteed.
+
+        :return: A string like:
+            'rebalance?dryrun=false&allow_capacity_estimation=false&concurrent_partition_movements_per_broker=5'
+            'state'
+            'stop_proposal_execution'
+        """
+        warnings.warn("This method is deprecated as of 0.2.0, as it needlessly recreates requests functionality. "
+                      "It may be removed entirely in future versions. "
+                      "Please use get_composed_params instead.",
+                      DeprecationWarning,
+                      stacklevel=2)
+        combined_parameter_to_value = self.get_composed_params()
+
         # If we have any mappings, urlencode them and return the full string
         if combined_parameter_to_value:
             return f"{self.name}?{urlencode(combined_parameter_to_value)}"
@@ -182,6 +203,7 @@ class AddBrokerEndpoint(AbstractEndpoint):
         CCParameter.ExcludedTopicsParameter,
         CCParameter.GoalsParameter,
         CCParameter.JSONParameter,
+        CCParameter.ReviewIDParameter,
         CCParameter.ReplicaMovementStrategiesParameter,
         CCParameter.SkipHardGoalCheckParameter,
         CCParameter.ThrottleRemovedBrokerParameter,
@@ -208,7 +230,8 @@ class AdminEndpoint(AbstractEndpoint):
         CCParameter.ConcurrentPartitionMovementsPerBrokerParameter,
         CCParameter.DisableSelfHealingForParameter,
         CCParameter.EnableSelfHealingForParameter,
-        CCParameter.JSONParameter
+        CCParameter.JSONParameter,
+        CCParameter.ReviewIDParameter,
     )
     argparse_properties = {
         'args': (name,),
@@ -247,6 +270,7 @@ class DemoteBrokerEndpoint(AbstractEndpoint):
         CCParameter.ExcludeRecentlyDemotedBrokersParameter,
         CCParameter.JSONParameter,
         CCParameter.ReplicaMovementStrategiesParameter,
+        CCParameter.ReviewIDParameter,
         CCParameter.SkipURPDemotionParameter,
         CCParameter.VerboseParameter
     )
@@ -266,7 +290,9 @@ class FixOfflineReplicasEndpoint(AbstractEndpoint):
     description = "Fixes the offline replicas in the cluster (kafka 1.1+ only)"
     http_method = "POST"
     can_execute_proposal = True
-    available_Parameters = tuple()
+    available_Parameters = (
+        CCParameter.ReviewIDParameter,
+    )
     argparse_properties = {
         'args': (name,),
         'kwargs': dict(aliases=[name.replace('_', '-')], help=description)
@@ -337,7 +363,8 @@ class PauseSamplingEndpoint(AbstractEndpoint):
     can_execute_proposal = False
     available_Parameters = (
         CCParameter.JSONParameter,
-        CCParameter.ReasonParameter
+        CCParameter.ReasonParameter,
+        CCParameter.ReviewIDParameter,
     )
     argparse_properties = {
         'args': (name,),
@@ -386,6 +413,7 @@ class RebalanceEndpoint(AbstractEndpoint):
         CCParameter.IgnoreProposalCacheParameter,
         CCParameter.JSONParameter,
         CCParameter.ReplicaMovementStrategiesParameter,
+        CCParameter.ReviewIDParameter,
         CCParameter.SkipHardGoalCheckParameter,
         CCParameter.UseReadyDefaultGoalsParameter,
         CCParameter.VerboseParameter
@@ -417,6 +445,7 @@ class RemoveBrokerEndpoint(AbstractEndpoint):
         CCParameter.GoalsParameter,
         CCParameter.JSONParameter,
         CCParameter.ReplicaMovementStrategiesParameter,
+        CCParameter.ReviewIDParameter,
         CCParameter.SkipHardGoalCheckParameter,
         CCParameter.ThrottleRemovedBrokerParameter,
         CCParameter.UseReadyDefaultGoalsParameter,
@@ -439,7 +468,8 @@ class ResumeSamplingEndpoint(AbstractEndpoint):
     can_execute_proposal = False
     available_Parameters = {
         CCParameter.JSONParameter,
-        CCParameter.ReasonParameter
+        CCParameter.ReasonParameter,
+        CCParameter.ReviewIDParameter,
     }
     argparse_properties = {
         'args': (name,),
@@ -447,6 +477,38 @@ class ResumeSamplingEndpoint(AbstractEndpoint):
     }
 
 
+class ReviewEndpoint(AbstractEndpoint):
+    name = "review"
+    description = "Create, approve, or discard reviews"
+    http_method = "POST"
+    can_execute_proposal = False
+    available_Parameters = (
+        CCParameter.ApproveParameter,
+        CCParameter.DiscardParameter,
+        CCParameter.JSONParameter,
+        CCParameter.ReasonParameter
+    )
+    argparse_properties = {
+        'args': (name,),
+        'kwargs': dict(help=description)
+    }
+
+
+class ReviewBoardEndpoint(AbstractEndpoint):
+    name = "review_board"
+    description = "View already-created reviews"
+    http_method = "GET"
+    can_execute_proposal = False
+    available_Parameters = (
+        CCParameter.JSONParameter,
+        CCParameter.ReviewIDsParameter
+    )
+    argparse_properties = {
+        'args': (name,),
+        'kwargs': dict(aliases=[name.replace('_', '-')], help=description)
+    }
+
+
 class StateEndpoint(AbstractEndpoint):
     name = "state"
     description = "Get the state of cruise control"
@@ -477,6 +539,7 @@ class StopProposalExecutionEndpoint(AbstractEndpoint):
     can_execute_proposal = False
     available_Parameters = (
         CCParameter.JSONParameter,
+        CCParameter.ReviewIDParameter,
     )
     argparse_properties = {
         'args': (name,),
@@ -484,6 +547,34 @@ class StopProposalExecutionEndpoint(AbstractEndpoint):
     }
 
 
+class TopicConfigurationEndpoint(AbstractEndpoint):
+    name = "topic_configuration"
+    description = "Update the configuration of the specified topics"
+    http_method = "POST"
+    can_execute_proposal = True
+    available_Parameters = (
+        CCParameter.AllowCapacityEstimationParameter,
+        CCParameter.ConcurrentLeaderMovementsParameter,
+        CCParameter.ConcurrentPartitionMovementsPerBrokerParameter,
+        CCParameter.DryRunParameter,
+        CCParameter.ExcludeRecentlyDemotedBrokersParameter,
+        CCParameter.ExcludeRecentlyRemovedBrokersParameter,
+        CCParameter.GoalsParameter,
+        CCParameter.JSONParameter,
+        CCParameter.SkipHardGoalCheckParameter,
+        CCParameter.SkipRackAwarenessCheckParameter,
+        CCParameter.ReplicaMovementStrategiesParameter,
+        CCParameter.ReplicationFactorParameter,
+        CCParameter.ReviewIDParameter,
+        CCParameter.TopicParameter,
+        CCParameter.VerboseParameter
+    )
+    argparse_properties = {
+        'args': (name,),
+        'kwargs': dict(aliases=[name.replace('_', '-')], help=description)
+    }
+
+
 class TrainEndpoint(AbstractEndpoint):
     name = "train"
     description = "Train the linear regression model"
